@@ -26,6 +26,7 @@ import zipfile
 # for type checking
 from collections.abc import Mapping
 from io import BytesIO, StringIO
+from typing import Union
 
 import numpy as np
 
@@ -470,7 +471,7 @@ def vector_to_spherical(cartesian):
 
 def spherical_to_vector(spherical):
     """
-    Convert a set of (n, 2) spherical vectors to (n, 3) vectors
+    Convert an array of `(n, 2)` spherical angles to `(n, 3)` unit vectors.
 
     Parameters
     ------------
@@ -489,8 +490,7 @@ def spherical_to_vector(spherical):
     theta, phi = spherical.T
     st, ct = np.sin(theta), np.cos(theta)
     sp, cp = np.sin(phi), np.cos(phi)
-    vectors = np.column_stack((ct * sp, st * sp, cp))
-    return vectors
+    return np.column_stack((ct * sp, st * sp, cp))
 
 
 def pairwise(iterable):
@@ -1412,7 +1412,9 @@ def type_named(obj, name):
     raise ValueError("Unable to extract class of name " + name)
 
 
-def concatenate(a, b=None):
+def concatenate(
+    a, b=None
+) -> Union["trimesh.Trimesh", "trimesh.path.Path2D", "trimesh.path.Path3D"]:  # noqa: F821
     """
     Concatenate two or more meshes.
 
@@ -2235,7 +2237,7 @@ def allclose(a, b, atol: float = 1e-8):
     bool indicating if all elements are within `atol`.
     """
     #
-    return float((a - b).ptp()) < atol
+    return float(np.ptp(a - b)) < atol
 
 
 class FunctionRegistry(Mapping):
@@ -2259,9 +2261,9 @@ class FunctionRegistry(Mapping):
 
     def __setitem__(self, key, value):
         if not isinstance(key, str):
-            raise ValueError("key must be a string, got %s" % str(key))
+            raise ValueError(f"key must be a string, got {key!s}")
         if key in self:
-            raise KeyError("Cannot set new value to existing key %s" % key)
+            raise KeyError(f"Cannot set new value to existing key {key}")
         if not callable(value):
             raise ValueError("Cannot set value which is not callable.")
         self._dict[key] = value
